builddata <- function(fname)
#read in the cleaned-up data and produce a data frame for model fitting
{
	cdata <- read.table(file=fname,header=T,sep="\t")

	current <- acomp(cdata,c("coral.1","algae.1","other.1"))
	original <- acomp(cdata, c("coral","algae","other"))
	df <- data.frame(reefid=cdata$reefid,year=cdata$year,long=cdata$long,lat=cdata$lat,c1=ilr(current)[,1],c2=ilr(current)[,2],o1=ilr(original)[,1],o2=ilr(original)[,2],sstlag=scale(cdata$sstlag,center=TRUE),depth=scale(cdata$depthavg,centre=TRUE),threat=factor(cdata$threat,levels=c("Low","Medium","High","Very High")),sst=scale(cdata$sst,center=TRUE),sstalag=scale(cdata$sstalag,center=TRUE),ssta=scale(cdata$ssta,center=TRUE),coastdistkm=scale(cdata$coastdistkm,center=TRUE),climlag=scale(cdata$sstlag - cdata$sstalag,center=TRUE),clim=scale(cdata$sst - cdata$ssta,center=TRUE))

	df$z1 <- df$c1-df$o1
	df$z2 <- df$c2-df$o2
	
	return(df)
}

plotraw <- function(fname,mono=TRUE,bgcol="white")
#plot the raw data from file fname
#open circle at start of series of consecutive observations on same reef
#filled circle at end
#if mono is TRUE, use grey scale, otherwise orange for end, blue for start
#bgcol: background colour
{
	cdata <- read.table(file=fname,header=T,sep="\t")
	plot.acomp(c(1,1,1),type="n",labels=c("coral","algae","other")) #dummy plot to create graph
       if(!(bgcol=="white")){
          s60 <- sin(pi/3)
          c60 <- cos(pi/3)
          s30 <- sin(pi/6)
          c30 <- cos(pi/6)
          xv <- c(0,c60,1,0)
          yv <- c(0,s60,0,0)
          polygon(xv,yv,col=bgcol)
      }        
	nreefs <- length(unique(cdata$reefid))
	print(paste("Number of reefs:",nreefs),quote=FALSE)
	ppoints <- plotrawpoints(cdata,drawlines=TRUE,mono=mono,bgcol=bgcol)
	plotrawpoints(cdata,drawlines=FALSE,mono=mono,bgcol=bgcol)

        legend(1,0.35,c("start","end"),bty="n",pch=c(1,16),col=rev(ppoints$colors))
        
	print(paste("Number of time series:",ppoints$nseries),quote=FALSE)
	print(paste("Number of pairs:",dim(cdata)[1]),quote=FALSE)
	print(paste("Median series length:",median(ppoints$slens,na.rm=TRUE)),quote=FALSE)
	srange <- range(ppoints$slens,na.rm=TRUE)
	print(sprintf("Range of series length: %d %d",srange[1],srange[2]))
	return(ppoints$slens)
}

plotrawpoints <- function(cdata,drawlines,mono,bgcol)
#if drawlines is FALSE, draw the points only
#if mono is TRUE, use grey scale, otherwise orange for end, blue for start
#bgcol is background colour: if not white, put lines in white    
#returns list containing nseries (number of time series) and slens (lengths of individual series), and colours used
{

	if(mono==TRUE){
		colors=c("black","black")
	} else {
		colors <-brewer.pal(5,"Dark2")[2:3]
                colors <- adjustcolor(colors,0.7) #gives transparency, hard to print as eps                
	}
        
	mygrey <- colors()[320]

      if(!(bgcol=="white")){
          linecol <- adjustcolor(colors()[237],0.7)#"white"
      } else linecol <- mygrey
        
	current <- acomp(cdata,c("coral.1","algae.1","other.1"))
	original <- acomp(cdata, c("coral","algae","other"))
	nseries <- 0
	slens <- array(dim=c(dim(cdata)[1],1))
	scount <- 1
	for(i in unique(cdata$reefid)){
	      oset <- original[cdata$reefid==i,]
	      cset <- current[cdata$reefid==i,]
 	      yset <- cdata$year[cdata$reefid==i]
	      dy <- diff(yset)
	      yd2 <- c(0,which(dy>1),length(dy)+1)
	      slen <- diff(yd2)+1
	      slens[scount:(scount+length(slen)-1)] <- slen
	      scount <- scount+length(slen)

	      nseries <- nseries + 1 + sum(dy>1)
	      if(length(yset)>1){#more than one pair
	          for(j in 1:length(yset)){
		  	if(j<length(yset))
				if(dy[j]>1){#break in the series
        			    plot.acomp(cset[j,],type="p",pch=16,add=TRUE,cex=0.5,col=colors[1])
				}
				if(j>1)
					if(dy[j-1]>1){#break in the series
					    plot.acomp(oset[j,],type="p",add=TRUE,cex=0.5,col=colors[2])
					    }
					    if(drawlines==TRUE){
						plot.acomp(rbind(oset[j,],cset[j,]),type="l",col=linecol,add=TRUE)
						}
					}
	      }
	      else{#only one pair
 	          if(drawlines==TRUE){
			plot.acomp(rbind(oset,cset),type="l",col=linecol,add=TRUE)
			}
	      }

	      if(is.null(dim(oset))){#only one pair
    	      plot.acomp(oset,type="p",add=TRUE,cex=0.5,col=colors[2])
    	      plot.acomp(cset,type="p",pch=16,add=TRUE,cex=0.5,col=colors[1])
  	  }
  	  else{
		plot.acomp(oset[1,],type="p",add=TRUE,cex=0.5,col=colors[2])
    		plot.acomp(cset[dim(cset)[1],],type="p",pch=16,add=TRUE,cex=0.5,col=colors[1])    
  	}
	}
	return(list(nseries=nseries,slens=slens,colors=colors))
}


plotperturb <- function(fname)
#plot the perturbing vectors from file fname
{
	mygrey <- colors()[320]
	cdata <- read.table(file=fname,header=T,sep="\t")
	current <- acomp(cdata,c("coral.1","algae.1","other.1"))
	original <- acomp(cdata, c("coral","algae","other"))
	plot(current-original,type="n",labels=c("coral","algae","other"))
	isoProportionLines(at=0.5,col=mygrey,labs=FALSE)
	plot(current-original,add=TRUE,pch=20,cex=0.75)
}

plotperturbbySST <- function(fname,bgcol="white")
#plot the perturbing vectors from file fname, coloured by lagged SST anomaly
#bgcol is background colour
#returns tpos, list of tick positions on colour scale from 0 to 1, and tlabs, the corresponding temperatures
{
	mygrey <- colors()[320]
   	cdata <- read.table(file=fname,header=T,sep="\t")
	current <- acomp(cdata,c("coral.1","algae.1","other.1"))
	original <- acomp(cdata, c("coral","algae","other"))

   	plot(current-original,type="n",labels=c("coral","algae","other"))

        if(!(bgcol=="white")){
          s60 <- sin(pi/3)
          c60 <- cos(pi/3)
          s30 <- sin(pi/6)
          c30 <- cos(pi/6)
          xv <- c(0,c60,1,0)
          yv <- c(0,s60,0,0)
          polygon(xv,yv,col=bgcol)
          isoProportionLines(at=0.5,col="white",labs=FALSE) #if grey background
      } else {
	isoProportionLines(at=0.5,col=mygrey,labs=FALSE)
      }

        colorind <- makecolorind(cdata$sstalag,cdata$sstalag)
        colorcode <- colorRamp(rev(brewer.pal(9,"RdBu")))
        mycolors <- adjustcolor(rgb(colorcode(colorind),max=255),0.7) #gives transparency, hard to print as eps
        mypch <- rep(16,dim(cdata)[1])
        mypch[cdata$threat=="Medium"] <- 17
        mypch[cdata$threat=="High"] <- 18
        mypch[cdata$threat=="Very High"] <- 18        
        plot(acomp(current)-acomp(original),add=TRUE,pch=mypch,cex=1,col=mycolors)

        legend(1,0.35,c("low threat","medium threat","high/very high threat"),bty="n",pch=c(16,17,18),col=rgb(colorcode(0),max=255))

        
        tlabs <- pretty(cdata$sstalag)
        tpos <- makecolorind(tlabs,cdata$sstalag)
        tlabs <- tlabs[tpos >= 0 & tpos <= 1]
        tpos <- tpos[tpos >= 0 & tpos <= 1]        
        return(list(tpos=tpos,tlabs=tlabs))
      }

makecolorind <- function(x,coldata)
#make colour range values
#x is some values (which should lie in the range of coldata)
#coldata is the reference scale, which will be adjusted so that its mean maps to 0.5, no value is outside [0,1], and the value furthest from mean will be at either 0 or 1 (depending on whether it's below or above the mean)
  {
        colorind <- (coldata-mean(coldata))
        cmax <- max(abs(colorind))
        colorind <- (x-mean(coldata))/(2*cmax)+0.5 #so that mean is at 0.5, and whole scale in [0,1], symmetrical around mean
        return(colorind)
  }


fitbasemodel <- function(df)
#fit the model we're using: composition, sst, depth, threat
{
	model <- lm(cbind(z1,z2) ~ cbind(o1,o2) + sstalag + climlag + threat, data=df)
	return(model)
}

testmodels <- function(df)
#compositional dynamics models: fit a model with all main effects, then remove each in turn.
#df is data frame
#prints out test results
#returns m2, the fitted model with all main effects
{

	m2 <- fitbasemodel(df=df)
	print(Anova(m2))

	#we want post-hoc tests for levels of threat (will have to do Bonferroni adjustment by hand)
	print("post-hoc, Medium threat")
	print(linearHypothesis(m2,"threatMedium=0",test="Pillai"))
	print("post-hoc, High threat")
	print(linearHypothesis(m2,"threatHigh=0",test="Pillai"))
	print("post-hoc, Very High threat")
	print(linearHypothesis(m2,"threatVery High=0",test="Pillai"))

	#we want post-hoc tests for the two columns of the composition effect A
	print("post-hoc, a_1 (effect of algae to coral)")
	print(linearHypothesis(m2, "cbind(o1, o2)o1=0",test="Pillai"))
	print("post-hoc, a_2 (effect of other to geometric mean of algae to coral)")
	print(linearHypothesis(m2, "cbind(o1, o2)o2=0",test="Pillai"))
	

	return(m2)
}

#plotting parameters from a compositional linear model with confidence ellipses on a ternary diagram
#need to have set up the plot already
#i is the index of the parameter (row of coef(model))
#model is the fitted model object
#level is the confidence level for the ellipse (e.g. 0.95))
#df is the degrees of freedom (e.g. 2)
#pch is the plot symbol
#color is the color
#lwd: line width for ellipse
#dopoint:plot point?
#doellipse: plot ellipse?
#adds a symbol to the existing plot with an approximate confidence ellipse
parameterellipse <- function(i,model,level,df,pch,color,lwd=1,dopoint=TRUE,doellipse=TRUE)
#note that there may be NA parameters in coef(model) (the parameters of the model). The corresponding rows and columns will be missing from vcov(model) (the covariance matrix of the parameters). This function should take care of that (and will plot nothing in such cases).
{
	if(!any(is.na(coef(model)[i,]))){
		ic <- ilrInv(coef(model))
		pr <- ic[i,]

		pname <- rownames(coef(model))[i]

		rl <- sqrt(qchisq(level,df=df))
		vaux <- vcov(model)
		plist <- which(!is.na(coef(model)[,1]))
		np <- length(plist)
		j <- sum(plist<=i) #there may be NA parameters in coef(model). These won't appear in the rows and cols of vcov(model)
		vparm <- ilrvar2clr(vaux[c(j,j+np),c(j,j+np)])
#		if(dopoint) plot.acomp(pr,pch=pch,add=TRUE,col=color,cex=0.75)
		if(dopoint){
                    plot.acomp(pr,pch=pch,add=TRUE,col=adjustcolor(color,0.7),cex=1.5)
                    plot.acomp(pr,pch=as.character(i),add=TRUE,col="white",cex=0.75)
                }
		if(doellipse) ellipses(acomp(pr),vparm,r=rl,col=color,lwd=lwd)
	}
}

varianceellipse <- function(model,color,lty,lwd=1)
#given a fitted model, draw an ellipse representing the noise covariance matrix Sigma with specified color and linetype. Ellipse is drawn at Mahalanobis distance of 1.
{
	vsigma <- ilrvar2clr(estVar(model))
	ellipses(acomp(c(1,1,1)),vsigma,col=color,lty=lty,lwd=lwd)
}

plotmodel <- function(model,makelegend=TRUE,whichparms=NULL,plotvariance=TRUE,bgcol="white",lx=1,ly=0.5,lwd=1)
#plot parameters of fitted model on a ternary plot
#makelegend TRUE if we want a legend
#whichparms (optional) indices of parameters we want to plot
#plotvariance (optional): show the unit Mahalanobis distance ellipse for covariance matrix?
#lx and ly are legend position (optional)    
{
	mygrey <- colors()[320]
	colors1 <-brewer.pal(5,"Dark2")
	colors2 <- brewer.pal(3,"BuPu")
        if(!(bgcol=="white")) intcol <- "grey" else intcol <- "black"
        cbPalette <- c("#56B4E9","#0072B2" , "#009E73", "#CC79A7","#F0E442","#E69F00","#D55E00")
        colors <- c(intcol,cbPalette)
        transcolors <- adjustcolor(colors,0.7)
	plot(acomp(c(1,1,1)),type="n",labels=c("coral","algae","other"))
        if(!(bgcol=="white")){
          s60 <- sin(pi/3)
          c60 <- cos(pi/3)
          s30 <- sin(pi/6)
          c30 <- cos(pi/6)
          xv <- c(0,c60,1,0)
          yv <- c(0,s60,0,0)
          polygon(xv,yv,col=bgcol)
          isoProportionLines(at=0.5,col="white",labs=FALSE) #if dark background
      } else {
	isoProportionLines(at=0.5,col=mygrey,labs=FALSE)
      }
        nparm <- dim(coef(model))[1]
        if(is.null(whichparms)) whichparms <- 1:nparm
	for(i in whichparms){
      	      parameterellipse(i=i,model=model,level=0.95,df=2,pch=16,color=transcolors[i],lwd=lwd,dopoint=FALSE,doellipse=TRUE)
	      }
	for(i in whichparms){
              parameterellipse(i=i,model=model,level=0.95,df=2,pch=16,color=colors[i],lwd=lwd,dopoint=TRUE,doellipse=FALSE) #make sure points on top
	      }

	if(plotvariance) varianceellipse(model,color=adjustcolor("grey",0.7),lty="dashed",lwd=lwd)

	if(makelegend){
            llabs <- c("intercept",expression(bold(a)[1]),expression(bold(a)[2]),"SST anomaly","SST climatology","medium threat","high threat","very high threat",expression(bold(Sigma)))
            lpchd <- c(rep(16,nparm),NA)
            lpch <- c(as.character(1:nparm),NA)
            llty <- c(rep(NA,nparm),"dashed")
            lcol <- c(colors,"grey")
            if(plotvariance) whichparms <- c(whichparms,nparm+1)
            legend(lx,ly,legend=llabs[whichparms],bty="n",pch=lpchd[whichparms],lty=llty[whichparms],col=adjustcolor(lcol[whichparms],0.7),pt.cex=1.5)
            legend(lx,ly,legend=rep(NA,length(whichparms)),bty="n",pch=lpch[whichparms],lty=llty[whichparms],col=c(rep("white",nparm),"grey"),pt.cex=0.75)
            #legend(lx,ly,llabs[whichparms],bty="n",pch=lpch[whichparms],lty=llty[whichparms],col=lcol[whichparms])            
	}
}

colorBarpoly <- function(colvals,zlim,parfig=c(0,1,0.835,1),parmar=c(1,4,4,5),tlabs=NULL,tpos=NULL,title=NULL){
    #for colour bars: colvals is an array of colours, zlim is scale limits
    #parfig and parmar are vectors for par(fig) and par(mar)
    #tlabs (optional) specifies tick labels
    #tpos (optional) specifies what these labels are
    #title (optional) is a title for the colour bar
    
	par(fig=parfig,new=TRUE)
        ntick <- 4
        if(is.null(tpos)){
            tpos <- pretty(zlim,n=ntick)
            tlat <- (tpos-min(tpos))/(diff(zlim))
        } else tlat <- tpos
        if(is.null(tlabs)){
            tlabs <- round(tpos,digits=0)
        }        
        par(mar=parmar)
        image(matrix(1:length(colvals),ncol=1),axes=F,col=colvals)
        axis(1,at=tlat,tick=F,labels=tlabs,line=-.5)
        if(!is.null(title)) text(0.5,5,title)
	par(fig=c(0,1,0,1),new=FALSE)
     }


#multivariate spline correlograms, given data frame (must include long, lat, year) and fitted model, and number of resampling iterations
plotspatialcorr <- function(dff,model,resamp){
    resids <- residuals(model)
    poolcorr <- spline.correlog(x=coordinates(dff)[,1],y=coordinates(dff)[,2],z=resids,resamp=resamp,latlon=TRUE,quiet=TRUE)
    par(mfrow=c(2,1))
    plotnicesplinecorr(poolcorr,main="A: pooled across years",xlab="Distance/km")
    years <- unique(dff$year)
    nyears <- length(years)
    cbPalette <- c("#56B4E9","#0072B2" , "#009E73", "#CC79A7","#F0E442","#E69F00","#D55E00")
    ltys <- c(rep("solid",length(cbPalette)),rep("dashed",length(cbPalette)))
    cbPalette <- rep(cbPalette,2)
    
    for(i in 1:nyears){
        subyear <- years[i]
        residsub <- resids[dff$year==subyear,]
        dfsub <- dff[dff$year==subyear,]
        c1 <- spline.correlog(x=coordinates(dfsub)[,1],y=coordinates(dfsub)[,2],z=residsub,resamp=resamp,latlon=TRUE,quiet=TRUE)
        if(i==1){
            plotnicesplinecorr(c1,main="B: year by year",xlab="Distance/km",col=cbPalette[i],lty=ltys[i])
        } else {
            plotnicesplinecorr(c1,add=TRUE,col=cbPalette[i],lty=ltys[i])
        }
    }
    legend("topleft",legend=years,col=cbPalette,lty=ltys,bty="n",cex=0.5,horiz=TRUE)
}

#the plot.spline.correlog function from ncf, adapted to allow more flexibility in axis labels, main title, line colours and styles
#add argument: if TRUE, just draw the lines on existing plot    
plotnicesplinecorr <- function(x, xmax = 0, text = TRUE, xlab="Distance",ylab="Correlation",main="",col="black",lty="solid",add=FALSE) 
{
    obj <- x
    xmax <- ifelse(xmax == 0, obj$max.distance, xmax)
    x <- round(obj$real$x, 1)
    y <- round(obj$real$y, 2)
    xul <- round(quantile(obj$boot$boot.summary$x.intercept, 
        probs = c(0.025, 0.975), na.rm = TRUE), 1)
    yul <- round(quantile(obj$boot$boot.summary$y.intercept, 
        probs = c(0.025, 0.975), na.rm = TRUE), 2)
    if(add==FALSE){
        plot(obj$real$predicted$x, obj$real$predicted$y, xlim = c(0, 
        xmax), ylim = c(-1, 1), type = "n", xlab = xlab, 
        ylab = ylab,main=main,cex.axis=0.75)
    }
    lines(obj$real$predicted$x, obj$real$predicted$y,col=col,lty=lty)
    lines(c(0, max(obj$real$predicted$x)), c(0, 0))
    if (!is.null(obj$boot$boot.summary$predicted$x)) {
        lines(obj$boot$boot.summary$predicted$x, obj$boot$boot.summary$predicted$y["0.025",],col=col,lty=lty )
        lines(obj$boot$boot.summary$predicted$x, obj$boot$boot.summary$predicted$y["0.975",],col=col,lty=lty)
        polygon(x=c(obj$boot$boot.summary$predicted$x,rev(obj$boot$boot.summary$predicted$x)),y=c(obj$boot$boot.summary$predicted$y["0.025",],rev(obj$boot$boot.summary$predicted$y["0.975",])),col=adjustcolor(col,0.1),border=NA)
    }
}

getsststats <- function(fname)
#get estimates of mean and standard deviation (and range) for lagged sst anomaly and climatology
{
	cdata <- read.table(file=fname,header=T,sep="\t")
	amu1 <- mean(cdata$sstalag)
	asigma1 <- sd(cdata$sstalag)
	arange <- range(cdata$sstalag)
        climlag=cdata$sstlag - cdata$sstalag
	cmu1 <- mean(climlag)
	csigma1 <- sd(climlag)
	crange <- range(climlag)
        
	return(list(amu1=amu1,asigma1=asigma1,arange=arange,cmu1=cmu1,csigma1=csigma1,crange=crange))
}

getB <- function(model)
#B matrix for model dynamics, where fitted model is of the form
#ilr p(t) = c + A*x(t) + epsilon(t) (model for perturbing vector)
#and we want to study the dynamics of
#x(t+1) = c + B*x(t) + epsilon(t)
#so that B = A + I
{
	A <- t(coef(model)[c("cbind(o1, o2)o1","cbind(o1, o2)o2"),])#transpose because we want to write x(t) as a column vector
	B <- A + diag(2)
	return(B)
}

getS2 <- function(model)
#get covariance matrix for stochastic term in dynamic model,
#treating annual mean SST anomaly as a sequence of iid N(0,1),
#independent of error term in fitted model
{
	Sigma <- estVar(model)
        b1 <- coef(model)["sstalag",]
	S2 <- Sigma + b1 %*% t(b1)
	return(S2)
}

buildc <- function(model,s2,mu2,sigma2,threat="Low")
#the part of the fitted model that does not depend on time or composition
#s2 is the mean climatology we want to study (in degrees)
#mu2 is the current mean climatology (in degrees)
#sigma2 is the standard deviation of climatology (in degrees)
#threat is threat level at which to evaluate: default is "Low" (possible values "Low", "Medium", "High", "VeryHigh")
{
	c <- coef(model)["(Intercept)",] + coef(model)["climlag",]*(s2-mu2)/sigma2
	tlev <- switch(threat,Low=c(0,0),Medium=coef(model)["threatMedium",],High=coef(model)["threatHigh",],VeryHigh=coef(model)["threatVery High",])
	c <- c+tlev
	return(c)
}

checkstability <- function(B)
#does the model have a stationary distribution?
{
	lamax <- max(abs(eigen(B)$values))
	return(list(isstable=lamax < 1,lamax=lamax))
}

getSigmastar <- function(model)
#get covariance of stationary distribution
{
	B <- getB(model=model)
	S2 <- getS2(model)
	I4 <- diag(4)
   	BB <- kronecker(B,B)
   	vecV <- solve(I4-BB)%*%as.vector(S2)
   	V <- matrix(vecV,nrow=2,ncol=2) #covariance of stationary distribution (in coordinates)
	return(V)
}

getmustar <- function(model,s2,mu2,sigma2,threat)
#get stationary mean under fitted model
#model is the fitted model
#s2 is the mean climatology we want to study
#mu2 is the current mean climatology
#sigma2 is the standard deviation of climatology
#threat is threat level ("Low","Medium","High","VeryHigh")
{
	B <- getB(model=model)
	c <- buildc(model=model,s2=s2,mu2=mu2,sigma2=sigma2,threat=threat)
	mustar <- solve(diag(2)-B,c)
	return(mustar)
}

getsensitivitySST <- function(model,mu2,sigma2,threat,measure,grid=10,gridstep=0.1)
#calculate sensitivity of stationary density to changes in mean climatology
#return values of sensitivity (afx) in requested measure at points in grix xgrid
#input model (fitted model)
#mu2 is current mean climatology
#sigma2 is standard deviation of climatology
#threat is threat level ("Low","Medium","High","VeryHigh")
#choose measure "Aitchison" or "Lebesgue"
#grid specifies the range of values of coordinates (-grid to grid in each dimension) at which we evaluate density sensitivity
#gridstep is step sizes on grid    
{
	mustar <- getmustar(model=model,s2=mu2,mu2=mu2,sigma2=sigma2,threat=threat) #stationary mean
	Sigmastar <- getSigmastar(model=model) #stationary covariance
	xgrid <- seq(from=-grid,to=grid,by=gridstep)
	nx <- length(xgrid)
	newgrid <- expand.grid(o1=xgrid,o2=xgrid) #regular grid on which to get contours
	fx <- dmvnorm(x=newgrid,mean=mustar,sigma=Sigmastar) #stationary density
	mustarsens <- getmustarsens(model=model,sigma2=sigma2) #sensitivity of stationary mean to SST
	D <- getD(Sigmastar=Sigmastar,mustarsens=mustarsens)
	afx <- xsens(model=model,x=newgrid,mustar=mustar,D=D,fx=fx)

	#compute Jacobian to get density wrt Lebesgue measure
	if(measure=="Lebesgue"){
		J <- LebesgueJacobian(newgrid)
    		afx <- afx*J #Lebesgue measure
	}
	else if(measure!="Aitchison"){
		stop(c("Measure ",measure," not recognized"))
	}
	return(list(afx=afx,xgrid=xgrid))
}

getmustarsens <- function(model,sigma2)
#sensitivity of stationary mean to changes in mean climatology
#sigma2 is standard deviation of climatology
{
	B <- getB(model=model)
	b2 <- coef(model)["climlag",]
	mustarsens <- solve(diag(2)-B,b2/sigma2)
	return(mustarsens)	
}

getD <- function(Sigmastar,mustarsens)
#D is (Sigmastar)^{-1}*sensitivity of stationary mean (mustarsens)
{
	D <- solve(Sigmastar,mustarsens)
	return(D)
}

xsens <- function(model,x,mustar,D,fx)
#sensitivity of density to mean climatology at points x (specified as an n x 2 matrix)
#mustar is stationary mean
#D is (Sigmastar)^{-1}*sensitivity of stationary mean
#fx is density at points x (as a vector)
{

	nn <- dim(x)[1]
	mustar <- matrix(rep(mustar,nn),nrow=nn,byrow=TRUE)
	x0 <- x - mustar
	alphai <- as.matrix(x0)%*%D
	afx <- fx*alphai #sensitivity of stationary density at grid point
	nx <- sqrt(nn)
	afx <- matrix(afx,nrow=nx,ncol=nx)
	return(afx)
}

LebesgueJacobian <- function(grid)
#Jacobian for transformation from Aitchison to Lebesgue measure, evaluated at the points in grid, a matrix of coordinates of points (each row is coordinates for a single point)
#returns a square matrix of values of Jacobian
{
	newinv <- ilrInv(grid)
	nn <- dim(newinv)[1]
	nx <- sqrt(nn)
	J <- array(dim=c(nn,1))
	for (i in 1:nn){
	    J[i] <- 1/(sqrt(3)*prod(newinv[i,])) #p129 in Pawlowski-Glahn and Buccianti: Jacobian for ilr
	}
    	J <- matrix(J,nrow=nx,ncol=nx)
	return(J)	
}

dobootstrap <- function(df,reps,s2,mu2,sigma2,threat,lthresh=1,gr=4,grst=0.1,dmes="Lebesgue",nlevels=20,trunc=50,maxcol,sstset,alphacoral,alphaalgae,sststats,xpoints,xplabs)
#wrapper function for bootstrap of stationary density
#returns an object containing all the bootstrap info
{
	bvec <- boot(df,bootstatdens,R=reps,grid=gr,gridstep=grst,s2=s2,mu2=mu2,sigma2=sigma2,threat=threat,measure=dmes,sstset=sstset,alphacoral=alphacoral,alphaalgae=alphaalgae)
	return(bvec)
}

bootstatdens <- function(d,i,grid=10,gridstep=0.1,s2,mu2,sigma2,threat,measure,sstset,alphacoral,alphaalgae)
#bootstrap a model with climatology and threat
#d is original data (must have variables c1, c2, o1, o2, sstlag, climlag, threat)
#i is indices (arguments needed by boot)
#density will be calculated from a range (-grid,grid) of values of coordinate (in steps of gridstep)
#s2 is the mean climatology
#mu2 is the current mean climatology
#sigma2 is the standard deviation of climatology
#threat is threat level ("Low","Medium","High","VeryHigh")
#measure is "Aitchison" or "Lebesgue" (for sensitivity of stationary density)
#sstset is vector of climatology values at which to calculate risks
#alphacoral is value of coral cover for which we want marginal cdf
#alphaalgae is value of algal cover for which we want 1 - marginal cdf
#returns a vector containing:
#stationary mean (NA if largest eigenvalue >= 1)
#largest eigenvalue
#statdens, stationary density on grid
{

 	xgrid <- seq(from=-grid,to=grid,by=gridstep)
   	newgrid <- expand.grid(o1=xgrid,o2=xgrid) #regular grid on which to get contours
	mboot <- fitbasemodel(df=d[i,])
	B <- getB(model=mboot)
	S2 <- getS2(model=mboot)
	cc <- buildc(model=mboot,s2=s2,mu2=mu2,sigma2=sigma2,threat=threat)
	isstable <- checkstability(B)
	if(!(isstable$isstable)){
	       mustar <- c(NA,NA)
      	       statdens <- rep(NA,dim(newgrid)[1])
               afx <- rep(NA,dim(newgrid)[1])
	       Ps <- rep(NA,2*length(sstset))
	} else {
	       Sigmastar <- getSigmastar(mboot)
	       mustar <- getmustar(model=mboot,s2=s2,mu2=mu2,sigma2=sigma2,threat=threat) #stationary mean
  	       statdens <- dmvnorm(newgrid,mustar,Sigmastar)
	       sSST <- getsensitivitySST(model=mboot,mu2=mu2,sigma2=sigma2,threat=threat,measure=measure,grid=grid,gridstep=gridstep)
	       afx <- as.vector(sSST$afx)
	       Pset <- getmarginals(model=mboot,sstset=sstset,mu2=mu2,sigma2=sigma2,threat=threat,alphacoral=alphacoral,alphaalgae=alphaalgae)
	       Ps <- as.vector(c(Pset$Pcoral,Pset$Palgae))
	}

	return(c(mustar,isstable$lamax,statdens,afx,Ps,as.vector(B),as.vector(S2),cc))
}

getmarginals <- function(model,sstset,mu2,sigma2,threat,alphacoral,alphaalgae)
#get marginal risks for range of climatology
#model is fitted model
#alphacoral is coral cover value of interest
#alphaalgae is algal cover value of interest
#sstset is vector of climatology values at which to evaluate risk of coral below alphacoral, and algae above alphaalgae
#mu2 is current mean climatology
#sigma2 is current standard deviation of climatology
#threat is threat level
{
	nsst <- length(sstset)
	Sigmastar <- getSigmastar(model)
	Pcoral <- array(dim=c(nsst,1))
	Palgae <- array(dim=c(nsst,1))
	for(i in 1:nsst){
	      mustar <- getmustar(model=model,s2=sstset[i],mu2=mu2,sigma2=sigma2,threat=threat)
              Pcoral[i] <- Py1(alpha=alphacoral,mu=mustar,Sigma=Sigmastar,lower=TRUE)$value
	      Palgae[i] <- Py2(alpha=alphaalgae,mu=mustar,Sigma=Sigmastar,lower=FALSE)$value
              #TEST CODE FOR TRACKING DOWN INTEGRATION PROBLEMS
              #pcoral <- try(Py1(alpha=alphacoral,mu=mustar,Sigma=Sigmastar,lower=TRUE)$value,silent=TRUE)
              #if(class(pcoral)=="try-error"){
              #    save(list=c("model","mustar","Sigmastar","alphacoral"),file="Py1error.RData")
              #    stop()
              #} else Pcoral[i] <- pcoral
              #palgae <- try(Py2(alpha=alphaalgae,mu=mustar,Sigma=Sigmastar,lower=FALSE)$value,silent=TRUE)
              #if(class(palgae)=="try-error"){
              #    save(list=c("model","mustar","Sigmastar","alphaalgae"),file="Py2error.RData")
              #    stop()
              #} else Palgae[i] <- palgae
              ###########################
          }
	return(list(Pcoral=Pcoral,Palgae=Palgae))
}

Py2 <- function(alpha,mu,Sigma,lower=TRUE)
#marginal cdf for Y_2, the second component of a 3-part composition with mean vector mu and covariance matrix Sigma in ilr coordinates
#if lower=TRUE, returns P(Y_2 <= alpha), otherwise P(Y_2 > alpha)
{
	minx1 <- getminx1(alpha)
        P <- try(integrate(imarginalpha2,lower=minx1,upper=Inf,mu=mu,Sigma=Sigma,alpha=alpha,rel.tol=1e-10,subdivisions=100L),silent=TRUE)
        if(class(P)=="try-error"){#not guaranteed to solve the problem
            P <- try(integrate(imarginalpha2,lower=minx1,upper=Inf,mu=mu,Sigma=Sigma,alpha=alpha,rel.tol=1e-20,subdivisions=100L),silent=TRUE)
        }
        
	if(lower==TRUE){
		P$value <- 1-P$value
	}
	return(P)
}

Py1 <- function(alpha,mu,Sigma,lower=TRUE)
#marginal cdf for Y_1, the first component of a 3-part composition with mean vector mu and covariance matrix Sigma in ilr coordinates
#if lower=TRUE, returns P(Y_1 <= alpha), otherwise P(Y_1 > alpha)
{
    maxx1 <- getmaxx1(alpha)
    P <- try(integrate(imarginalpha1,lower=-Inf,upper=maxx1,mu=mu,Sigma=Sigma,alpha=alpha,rel.tol=1e-10,subdivisions=100L),silent=TRUE)
    if(class(P)=="try-error"){#not guaranteed to solve the problem
        P <- try(integrate(imarginalpha1,lower=-Inf,upper=maxx1,mu=mu,Sigma=Sigma,alpha=alpha,rel.tol=1e-20,subdivisions=100L),silent=TRUE)
    }

    if(lower==TRUE){
        P$value <- 1-P$value
    }
    return(P)
}


lalpha1 <- function(alpha,x1)
#upper limit on x2 for y2 >= alpha, given x1
{
	ly <- exp(-sqrt(2)*x1)
	x2 <- -2/sqrt(6) * log( (alpha*sqrt(ly)) / (1-alpha*(1+ly)) )
	return(x2)
}

lalpha2 <- function(alpha,x1)
#upper limit on x2 for y1 >= alpha, given x1
{
	ly <- exp(sqrt(2)*x1)
	x2 <- -2/sqrt(6) * log( (alpha*sqrt(ly)) / (1-alpha*(1+ly)) )
	return(x2)
}

cmarginalpha2 <- function(x1,mu,Sigma,q)
#P(X_2 <= q | X_1 = x_1), where q is chosen such that Y_1 or Y_2 >= alpha, where Y is ilrInv(X)
{
	s1 <- sqrt(Sigma[1,1])
	s2 <- sqrt(Sigma[2,2])
	rho <- Sigma[2,1]/(s1*s2)
	cmu <- mu[2]+s2/s1*rho*(x1-mu[1])
	csd <- sqrt((1-rho^2))*s2
	cx2 <- pnorm(q=q,mean=cmu,sd=csd)
	return(cx2)
}

umargin1 <- function(x1,mu,Sigma)
#unconditional marginal density of x1
{
	s1 <- sqrt(Sigma[1,1])
	fx1 <- dnorm(x=x1,mean=mu[1],sd=s1)
	return(fx1)
}


imarginalpha2 <- function(x1,mu,Sigma,alpha)
#integrate conditional marginal cdf of x2 <= lalpha1 over x1, where lalpha1 is chosen such that Y2 >= alpha, where Y = ilrInv(X)
{
	q <- lalpha1(alpha=alpha,x1=x1)
	cx2 <- cmarginalpha2(x1=x1,mu=mu,Sigma=Sigma,q=q)
	fx1 <- umargin1(x1=x1,mu=mu,Sigma=Sigma)
	return(cx2*fx1)
}

imarginalpha1 <- function(x1,mu,Sigma,alpha)
#integrate conditional marginal cdf of x1 <= lalpha1 over x1, where lalpha1 is chosen such that Y1 >= alpha, where Y = ilrInv(X)
{
	q <- lalpha2(alpha=alpha,x1=x1)
	cx2 <- cmarginalpha2(x1=x1,mu=mu,Sigma=Sigma,q=q)
	fx1 <- umargin1(x1=x1,mu=mu,Sigma=Sigma)
	return(cx2*fx1)
}

getminx1 <- function(alpha)
#min value of x1 for Y_2 >= alpha
{
	minx1 <- -log(1/alpha - 1)/sqrt(2)
	return(minx1)
}

getmaxx1 <- function(alpha)
#max value of x1 for Y_1 >= alpha
{
	maxx1 <- log(1/alpha - 1)/sqrt(2)
	return(maxx1)
}

extractboot <- function(bvec,grid,gridstep,lthresh,sstset)
#extract variables from bootstrap output
#bvec is the output from boot
#grid is outer limit of grid used in call to bootstatdens
#gridstep is size of step between grid points used in call to bootstatdens
#lthresh is eigenvalue threshold at which we discard reps (1 is sensible choice)
#sstset is vector of sst values at which we evaluated marginal risks
{
	ngrid <- length(seq(from=-grid,to=grid,by=gridstep))
	mustar <- bvec$t[,1:2]
	lamax <- bvec$t[,3]
	iNA <- lamax<lthresh #the stable replicates
	statdens <- bvec$t[iNA,4:(ngrid^2+3)]
	afx <- bvec$t[iNA,(ngrid^2+4):(2*ngrid^2+3)]
	nsst <- length(sstset)
	Pcoral <- bvec$t[iNA,(2*ngrid^2+4):(2*ngrid^2+3+nsst)]
	Palgae <- bvec$t[iNA,(2*ngrid^2+4+nsst):(2*ngrid^2+3+2*nsst)]
	B <- bvec$t[,(2*ngrid^2+4+2*nsst):(2*ngrid^2+7+2*nsst)]
	S2 <- bvec$t[,(2*ngrid^2+8+2*nsst):(2*ngrid^2+11+2*nsst)]
	cc <- bvec$t[,(2*ngrid^2+12+2*nsst):(2*ngrid^2+13+2*nsst)]
	return(list(mustar=mustar,lamax=lamax,iNA=iNA,statdens=statdens,afx=afx,Pcoral=Pcoral,Palgae=Palgae,B=B,S2=S2,cc=cc))
}

plot2marginalSSTboots <- function(bres,bres2,sstset,alphacoral,alphaalgae,mu2,sststats,xpoints,xplabs,labs,truncatesst=NA)
#plot bootstrapped marginal risks against SST
#for two sets of results with same SST info
#bres and bres2 are two objects returned by extractboot
#sstset is set of SST values used
#alphacoral is coral cover value of interest
#alphaalgae is algal cover value of interest
#mu1 is current mean climatology
#sststats is summary statistics for current SST, produced by getsststats
#xpoints and xplabs are extra points to mark on x axis, and their labels
#labs are a vector of labels to append to panel letters
#truncatesst: if NA (default), use the full range of sstset. If a number, truncate at this value above mean observed climatology.    
{
	gbc <- getbootrisk(bres$Pcoral)
	gba <- getbootrisk(bres$Palgae)

	gbc2 <- getbootrisk(bres2$Pcoral)
	gba2 <- getbootrisk(bres2$Palgae)


	par(mfrow=c(2,2),mar=c(4,4.2,2,1),oma=c(0,0,0,0))
	mygrey <- colors()[320]

	yl <- bquote(italic(P)~"(coral cover"<= ~ .(alphacoral) ~ ")")
	xl <-range(c(range(sstset),sststats$crange))
        if(!is.na(truncatesst)){
            xl[2] <- min(xl[2],sststats$cmu1+truncatesst)
        }
	plot(sstset,gbc$bmean,type="l",xlim=xl,ylim=c(0,1),xlab=expression(paste("Climatology (",degree,"C)")),ylab=yl)
	abline(v=mu2,lty="dotted")
	lines(x=sststats$crange,y=c(1,1),lwd=2,col=mygrey)#range of observed climatology
	lines(sstset,gbc$b025,lty="dashed")
	lines(sstset,gbc$b975,lty="dashed")
	axis(side=1,at=xpoints,labels=xplabs)
	text(min(sstset),0.925,paste("A: ",labs[1]),adj=c(0,NA))

	plot(sstset,gbc2$bmean,type="l",xlim=xl,ylim=c(0,1),xlab=expression(paste("Climatology (",degree,"C)")),ylab=yl)
	abline(v=mu2,lty="dotted")
	lines(x=sststats$crange,y=c(1,1),lwd=2,col=mygrey)#range of observed climatology
	lines(sstset,gbc2$b025,lty="dashed")
	lines(sstset,gbc2$b975,lty="dashed")
	axis(side=1,at=xpoints,labels=xplabs)
	text(min(sstset),0.925,paste("B: ",labs[2]),adj=c(0,NA))
	
	yl <- bquote(italic(P)~"(algal cover"> ~ .(alphaalgae) ~ ")")
	plot(sstset,gba$bmean,type="l",xlim=xl,ylim=c(0,1),xlab=expression(paste("Climatology (",degree,"C)")),ylab=yl)
	abline(v=mu2,lty="dotted")
	lines(x=sststats$crange,y=c(1,1),lwd=2,col=mygrey)#range of observed climatology
	lines(sstset,gba$b025,lty="dashed")
	lines(sstset,gba$b975,lty="dashed")
	axis(side=1,at=xpoints,labels=xplabs)
	text(min(sstset),0.925,paste("C: ",labs[3]),adj=c(0,NA))
	
	plot(sstset,gba2$bmean,type="l",xlim=xl,ylim=c(0,1),xlab=expression(paste("Climatology (",degree,"C)")),ylab=yl)
	abline(v=mu2,lty="dotted")
	lines(x=sststats$crange,y=c(1,1),lwd=2,col=mygrey)#range of observed climatology
	lines(sstset,gba2$b025,lty="dashed")
	lines(sstset,gba2$b975,lty="dashed")
	axis(side=1,at=xpoints,labels=xplabs)
	text(min(sstset),0.925,paste("D: ",labs[4]),adj=c(0,NA))
}

getbootrisk <- function(bootrisk)
#bootrisk is an array extracted from bootstrap results (e.g. bres$Pcoral)
#returns bootstrap mean
{
	
	bmean <- colMeans(bootrisk)
	bq <- apply(bootrisk, 2, quantile, probs = c(0.025,0.975))
	return(list(bmean=bmean,b025=bq[1,],b975=bq[2,]))
}

bootsenspoly <- function(bres,grid,gridstep,truncv,sfunc){
#get polgons of sensitivity out of the object returned by extractboot
#grid and gridstep are the end value and step size for the grid on which we want sensitivity
#truncv is max abs value we want
#sfunc is summary function we want ("mean" or "sd")

	xgrid <- seq(from=-grid,to=grid,by=gridstep)
	newgrid <- expand.grid(o1=xgrid,o2=xgrid) #regular grid on which to get contours
	sens <- apply(bres$afx,2,sfunc)
	sensgrid <- matrix(sens,nrow=length(xgrid),ncol=length(xgrid))
        gp <- getgrid(grid,gridstep) #corners of polygons
        myp <- comptoxy(ilrInv(gp))

	if(sfunc=="mean")
	{
            colorcode <- colorRamp(rev(brewer.pal(9,"RdBu")))
            afxt <- truncate(sens,truncv)
            colorind <- makecolorindpoly(afxt,c(-truncv,truncv))
            np <- length(afxt)
            for(i in 1:np){
                sp <- (i-1)*6+1
                ep <- sp+4
                mcol <- colorind[i]
                polygon(myp$x[sp:ep],myp$y[sp:ep],col=rgb(colorcode(mcol),max=255),border=NA)
            }
            clines <- contourLines(xgrid,xgrid,sensgrid,levels=0)#we want the zero contour
            for(i in 1:length(clines)){
                cback <- ilrInv(cbind(clines[[i]]$x,clines[[i]]$y))
                cl <- clines[[i]]$level
                plot(cback,type="l",col="black",add=TRUE)
            }
	} else if(sfunc=="sd")
	{
            colorcode <- colorRamp(brewer.pal(9,"Greys"))
            afxt <- truncate(sens,truncv)
            colorind <- makecolorindpolyseq(afxt,c(0,truncv))
            np <- length(afxt)
            for(i in 1:np){
                sp <- (i-1)*6+1
                ep <- sp+4
                mcol <- colorind[i]
                polygon(myp$x[sp:ep],myp$y[sp:ep],col=rgb(colorcode(mcol),max=255),border=NA)
            }
	} 	else{
		stop(c("sfunc ",sfunc," not recognized"))
  	}

}

getgrid <- function(grid,gridstep)
#build convenient grid for plotting polygons
    {
	xgrid <- seq(from=-grid,to=grid,by=gridstep)
        grd <- expand.grid(o1=xgrid,o2=xgrid)
        gp <- (dim(grd))[1]
        x <- grd[,1]
        y <- grd[,2]
        cr <- 1
        gp <- array(dim=c(length(x)*6,2))
        for(i in 1:length(x)){
            gp[cr,1] <- x[i]-gridstep/2
            gp[cr,2] <- y[i]-gridstep/2
            gp[cr+1,1] <- x[i]-gridstep/2
            gp[cr+1,2] <- y[i]+gridstep/2
            gp[cr+2,1] <- x[i]+gridstep/2
            gp[cr+2,2] <- y[i]+gridstep/2
            gp[cr+3,1] <- x[i]+gridstep/2
            gp[cr+3,2] <- y[i]-gridstep/2
            gp[cr+4,1] <- x[i]-gridstep/2
            gp[cr+4,2] <- y[i]-gridstep/2
            gp[cr+5,1] <- NA
            gp[cr+5,2] <- NA
            cr <- cr+6
        }
        return(gp)
    }

comptoxy <- function(z)
#turn a composition into x and y coordinates that we can use with ordinary plotting commands, but that match up with plot.acomp
#input z, a composition
#output a list of x and y coordinates
    {
        s60 <- sin(pi/3)
        c60 <- cos(pi/3)
        x <- z[,2]+z[,3]*c60
        y <- z[,3]*s60
        return(list(x=x,y=y))
    }

truncate <- function(x,truncv)
#truncate vector x: values below -truncv set to -truncv, above truncv set to truncv
    {
        x[x > truncv] <- truncv
        x[x < -truncv] <- -truncv
        return(x)
}

makecolorindpoly <- function(x,coldata)
#make colour range values for diverging scale
#x is some values (which should lie in the range of coldata)
#coldata is the reference scale, which will be adjusted so that 0 maps to 0.5, no value is outside [0,1], and the value furthest from 0 will be at either 0 or 1 (depending on whether it's below or above zero)
#trunc: any absolute values greater than trunc will be set to trunc
  {
      cmax <- max(abs(coldata),na.rm=TRUE)
      colorind <- x/(2*cmax)+0.5 #so that 0 maps to 0.5, and whole scale in [0,1], symmetrical around 0
      return(colorind)
  }

colorBarfor2poly <- function(zlim,pos,colorcode=colorRamp(rev(brewer.pal(9,"RdBu")))){
    #wrapper to position colour bar on multipanel figures
    #colorcode is the colour palette used
	if(pos==1){
		parfig <- c(0,0.5,0.7,0.85)
	} else if(pos==2){
		parfig <- c(0.5,1,0.7,0.85)
	} else if(pos==-1){
		parfig <- c(0.5,1,0.8,0.975)
	} else if(pos==-2){
		parfig <- c(0.5,1,0.4,0.575)
	}
 #       colorcode <- colorRamp(rev(brewer.pal(9,name)))
        cl <- seq(0,1,length.out=100)
        colorlevels <- rgb(colorcode(cl),max=255)
        colorBarpoly(colvals=colorlevels,zlim=zlim,parfig=parfig)
    }

makecolorindpolyseq <- function(x,coldata)
#make colour range values for sequential scale
#x is some values (which should lie in the range of coldata)
#coldata is the reference scale, which will be adjusted so that no value is outside [0,1], and the value furthest from 0 will be at either 0 or 1 (depending on whether it's below or above zero)
#trunc: any absolute values greater than trunc will be set to trunc
  {
      cmax <- max(abs(coldata),na.rm=TRUE)
      colorind <- x/cmax
      return(colorind)
  }

bootdenspoly <- function(bres,grid,gridstep,measure,truncv,sfunc){
#get polygons of stationary density out of the object returned by extractboot (bres)
#grid and gridstep are the end value and step size for the grid on which we want density
#measure is "Aitchison" or "Lebesgue"
#truncv is max abs value we want
#sfunc is summary function we want ("mean" or "sd")

	xgrid <- seq(from=-grid,to=grid,by=gridstep)
	newgrid <- expand.grid(o1=xgrid,o2=xgrid) #regular grid on which to get contours
	statdens <- apply(bres$statdens,2,sfunc)
	densgrid <- matrix(statdens,nrow=length(xgrid),ncol=length(xgrid))
        gp <- getgrid(grid,gridstep) #corners of polygons
        myp <- comptoxy(ilrInv(gp))
	    
	#compute Jacobian to get density wrt Lebesgue measure
	if(measure=="Lebesgue"){
		J <- LebesgueJacobian(newgrid)
		z <- densgrid*J
	}
  	else if(measure=="Aitchison"){
	     z <- densgrid
  	}
  	else{
		stop(c("Measure ",measure," not recognized"))
  	}
        colorcode <- colorRamp(brewer.pal(9,"Greys"))
        dt <- truncate(z,truncv)
        colorind <- makecolorindpolyseq(dt,c(0,truncv))
        np <- length(dt)
        for(i in 1:np){
            sp <- (i-1)*6+1
            ep <- sp+4
            mcol <- colorind[i]
            polygon(myp$x[sp:ep],myp$y[sp:ep],col=rgb(colorcode(mcol),max=255),border=NA)
        }
    }

colorBarfor6poly <- function(zlim){
    #wrapper to position colour bar on multipanel figures
    #only implemented for color bar below first panel
    parfig <- c(0.025,0.475,0.645,0.75)
    colorcode <- colorRamp(brewer.pal(9,"Greys"))
    cl <- seq(0,1,length.out=100)
    colorlevels <- rgb(colorcode(cl),max=255)
    colorBarpoly(colvals=colorlevels,zlim=zlim,parfig=parfig)
    }

colorBarfor4poly <- function(zlim,pos){
    #wrapper to position colour bar on multipanel figures
	if(pos==1){
		parfig <- c(0,0.5,0.52,0.67)
	} else if(pos==1.5){ 
		parfig <- c(0.25,0.75,0.52,0.67)
	}else if(pos==2){
		parfig <- c(0,0.5,0.02,0.17)
	} else if(pos==2.5){
 		parfig <- c(0.25,0.75,0.02,0.17)
	} else if(pos==3){
		parfig <- c(0.5,1,0.52,0.67)
	} else if(pos==4){
	   	parfig <- c(0.5,1,0.02,0.17)
	}
        colorcode <- colorRamp(brewer.pal(9,"Greys"))
        cl <- seq(0,1,length.out=100)
        colorlevels <- rgb(colorcode(cl),max=255)
        colorBarpoly(colvals=colorlevels,zlim=zlim,parfig=parfig)
    }


ploteigen <- function(bres)
#plot eigenvalues of bootstrap data bres in complex plane
#first eigenvalue in blue, second in red, one point for each bootstrap rep
{
	theta <- seq(from=0,to=2*pi,length.out=1e3)
	x <- cos(theta)
	y <- sin(theta)
	plot(x,y,type="l",xlab="Real",ylab="Imaginary",asp=1)
		
	reps <- dim(bres$B)[1]
	c1 <- 0
	c2 <- 0
	for(i in 1:reps){
    	      B <- matrix(bres$B[i,],nrow=2,ncol=2)
	      ev <- as.complex(eigen(B)$values)
	      points(Re(ev[1]),Im(ev[1]),col="blue")
	      points(Re(ev[2]),Im(ev[2]),col="red")	      
	      if(abs(Im(ev[1]))>0){
		c1 <- c1 + 1
	      }
	      if(abs(Im(ev[2]))>0){
		c2 <- c2 + 1
	      }

	}
	print(paste("Proportion reps with complex eigenvalue 1:",c1/reps),quote=FALSE)
	print(paste("Proportion reps with complex eigenvalue 2:",c2/reps),quote=FALSE)
}

getproportionstable <- function(bres)
{
	pstab <- sum(bres$lamax<1)/length(bres$lamax)
	lmean <- mean(bres$lamax)
	lsd <- sd(bres$lamax)
	return(list(pstab=pstab,lmean=lmean,lsd=lsd))
}

SSTequivalent <- function(model,sigma2)
#what is medium vs low threat worth in degrees heating of climatology?    
#input a fitted model and the standard deviation of climatology
#return a list containing ssteqshort (how many degrees heating medium threat is worth in short term), cos2thetashort (what proportion of medium vs low threat effect is in direction of climatology effect in short term), ssteqlong (how many degrees heating worth in long term) and cos2thetalong (what proportion of medium vs low threat effect is in direction of climatology in long term)
  {
  #in short term?
    b1 <- coef(model)["climlag",]
    mthreat <- coef(model)["threatMedium",]
    ssteqshort <- sigma2*dotprod(mthreat,b1)/sqnormx(b1)
    cos2thetashort <- (dotprod(mthreat,b1)/(normx(mthreat)*normx(b1)))^2
#and in long term?
    d <- getmustarsens(model,sigma2)
    B <- getB(model=model)
    e <- solve(diag(2)-B,mthreat)
    ssteqlong <- dotprod(e,d)/sqnormx(d)
    cos2thetalong <- (dotprod(e,d)/(normx(e)*normx(d)))^2
    return(list(ssteqshort=ssteqshort,cos2thetashort=cos2thetashort,ssteqlong=ssteqlong,cos2thetalong=cos2thetalong))
  }

climanom <- function(model,sigma1,sigma2)
#if long-term effect is described by anomaly rather than climatology parameter, how different would things be?
#input a fitted model and the standard deviations of anomaly (sigma1) and of climatology (sigma2)
#return a list containing cos2theta (what proportion of long-term anomaly effect is captured by projection onto climatology?) and climanomeq (if we were to use anomaly effect instead of climatology, how many degrees under original model is a degree of long-term warming worth?)
  {
    b1 <- coef(model)["sstalag",]      
    b2 <- coef(model)["climlag",]
    f <- getmustarsens(model,sigma2)
    B <- getB(model=model)
    g <- solve(diag(2)-B,b1/sigma1)
    climanomeq <- dotprod(f,g)/sqnormx(f)
    cos2theta <- (dotprod(f,g)/(normx(f)*normx(g)))^2
    return(list(climanomeq=climanomeq,cos2theta=cos2theta))
  }


#dot product of vectors
dotprod <- function(x,y){
    return(sum(x*y))
}

#norm of vector
normx <- function(x){
    return(sqrt(sum(x^2)))
}

#squared norm of vector
sqnormx <- function(x){
    return(sum(x^2))
}


bootSSTeq <- function(d,i,sigma2)
#function for booting SST equivalent  
#d is original data (must have variables c1, c2, o1, o2, sstalag, climlag, threat)
#i is indices (arguments needed by boot)
#sigma2 is the standard deviation of climatology
#returns a vector of list objects from SSTequivalent
{
  mboot <- fitbasemodel(df=d[i,])
  ssteq <- SSTequivalent(model=mboot,sigma2=sigma2)
  return(c(ssteq$ssteqshort,ssteq$cos2thetashort,ssteq$ssteqlong,ssteq$cos2thetalong))
}

bootclimanom <- function(d,i,sigma1,sigma2)
#function for booting climatology equivalent of long-term anomaly effect  
#d is original data (must have variables c1, c2, o1, o2, sstalag, climlag, threat)
#i is indices (arguments needed by boot)
#sigma1 is standard deviation of anomaly    
#sigma2 is the standard deviation of climatology
#returns a vector of list objects from climanom
{
  mboot <- fitbasemodel(df=d[i,])
  clan <- climanom(model=mboot,sigma1=sigma1,sigma2=sigma2)
  return(c(clan$climanomeq,clan$cos2theta))
}


SSTdirectionplot <- function(model,s2,mu2,sigma2){
    mss <- getmustarsens(model,sigma2) #sensitivity of stationary mean to SST
    mustar <- getmustar(model=model,s2=s2,mu2=mu2,sigma2=sigma2,threat="Low")
    nsteps <- 100
    mssm <- matrix(rep(mss,nsteps),nrow=nsteps,ncol=2,byrow=TRUE)
    mustarm <- matrix(rep(mustar,nsteps),nrow=nsteps,ncol=2,byrow=TRUE)
    sstseq <- seq(from=0,to=3.5,length.out=nsteps)
    mustarchange <- mustarm+sstseq*mssm
    b1 <- coef(mgbr)["climlag",]
    B <- getB(mgbr)
    cchange <- rbind(c(1/3,1/3,1/3),ilrInv(b1*sstseq[nsteps]/sigma2))
    s60 <- sin(pi/3)
    c60 <- cos(pi/3)
 
    par(mar=c(0,2,0,2))
    par(xpd=TRUE)
    plot.acomp(c(1,1,1),type="n",labels=c("coral","algae","other")) #dummy plot to create graph
    mygrey <- colors()[320]
    isoProportionLines(at=0.5,col=mygrey)

    x <- cchange[,2]+cchange[,3]*c60
    y <- cchange[,3]*s60
    arrows(x0=x[1],y0=y[1],x1=x[2],y1=y[2],lwd=3,length=0.1)
    plot.acomp(ilrInv(rbind(mustar,mustarchange)),type="l",lwd=3,add=TRUE,lty="dashed")
    msc <- ilrInv(mustarchange)
    x <- msc[,2]+msc[,3]*c60
    y <- msc[,3]*s60
    nx <- length(x)
    arrows(x0=x[nx-1],y0=y[nx-1],x1=x[nx],y1=y[nx],lwd=3,length=0.1)
    legend("topleft",bty="n",lwd=3,lty=c("solid","dashed"),legend=c(expression(short-term:~frac(d~bold(c),d~italic(u[c]))==frac(1,italic(s[c]))~bold(beta)[2]),expression(long-term:~frac(d~bold(mu)^"*",d~italic(u[c]))==frac(1,italic(s[c]))~(bold(I)-bold(B))^-1~bold(beta)[2])),horiz=TRUE,seg.len=2.5)
}
    
simulateonemulti <- function(nseries,slens,inits,npoints,model,B,Sigma,df){
#simulate a single realization from a given model, assuming multinomial observation error
#nseries: number of series of consecutive observations
#slens: length of each series
#inits: initial ilr-transformed composition in each series (nseries x 2 array)
#npoints: number of points sampled
#model: model from which we want to simulate (uses sstlag, centred and scaled)
#B: dynamics matrix in process equation
#Sigma: process error covariance (shouldn't include effects of SST anomaly)
#df: real data frame for which we want a corresponding simulated data frame
#returns simulated data frame with same structure (and using observed values of environmental variables)
    dfsim <- df
    dfsim$o1 <- NA
    dfsim$o2 <- NA
    dfsim$c1 <- NA
    dfsim$c2 <- NA
    rowind <- 1
    for(i in 1:nseries){
        xt <- c(inits[i,1],inits[i,2])
        mobst <- rmultinom(n=1,size=npoints,prob=ilrInv(xt))
        yt <- ilr(t(mobst))

        
        dfsim$o1[rowind] <- yt[1]
        dfsim$o2[rowind] <- yt[2]
        for(tt in 2:slens[i]){
            tlev <- as.character(dfsim$threat[rowind])
            if(tlev=="Very High") tlev <- "VeryHigh" #needed in this form in buildc
            nu <- mvrnorm(n=1,mu=c(0,0),Sigma=Sigma)
            c <- buildc(model=model,s2=dfsim$climlag[rowind],mu2=0,sigma2=1,threat=tlev)
            b1 <- coef(model)["sstalag",]
            xtplus1 <- c + B%*%xt + nu + b1*dfsim$sstalag[rowind]
            mobstplus1 <- rmultinom(n=1,size=npoints,prob=ilrInv(t(xtplus1)))
            ytplus1 <- ilr(t(mobstplus1))

            rowind <- rowind+1
            dfsim$c1[rowind-1] <- ytplus1[1]
            dfsim$c2[rowind-1] <- ytplus1[2]
            if(tt < slens[i]){
                dfsim$o1[rowind] <- ytplus1[1]
                dfsim$o2[rowind] <- ytplus1[2]
            }
            xt <- xtplus1
            yt <- ytplus1
        }
    }
    dfsim$z1 <- dfsim$c1-dfsim$o1
    dfsim$z2 <- dfsim$c2-dfsim$o2
    return(dfsim)
 }

getinits <- function(cdata)
#get number of series, length of series, and observed initial value in each series
#data frame cdata includes c1, c2: ilr-transformed composition at time t+1
#o1,o2: ilr-transformed composition at time t
#reefid: reef identifiers
#year: calendar year
{
    nseries <- 0
    slens <- numeric(0)
    scount <- 1
    inits <- array(dim=c(0,2))
    for(i in unique(cdata$reefid)){
        yset <- cdata$year[cdata$reefid==i]
        oset <- cdata[cdata$reefid==i,c("o1","o2")]
        dy <- diff(yset)
        yd2 <- c(0,which(dy>1),length(dy)+1)
        slen <- diff(yd2)+1
        slens <- c(slens,slen)
        scount <- scount+length(slen)
        nseries <- nseries + 1 + sum(dy>1)
        inits <- rbind(inits,oset[c(1,which(dy>1)+1),])
    }
    return(list(nseries=nseries,slens=slens,inits=inits))
}

addpoint <- function(i,model,df,pch,color)
#plot point for the ith parameter estimate    
#note that there may be NA parameters in coef(model) (the parameters of the model). The corresponding rows and columns will be missing from vcov(model) (the covariance matrix of the parameters). This function should take care of that (and will plot nothing in such cases).
{
	if(!any(is.na(coef(model)[i,]))){
		ic <- ilrInv(coef(model))
		pr <- ic[i,]
		plot.acomp(pr,pch=pch,add=TRUE,col=adjustcolor(color,0.3),cex=0.5)
	}
}

addpointcoord <- function(x,pch,color)
#plot point at ilr coordinates x
{
    pr <- ilrInv(x)
#    plot.acomp(pr,pch=pch,add=TRUE,col=adjustcolor(color,0.7),cex=0.75)
    plot.acomp(pr,pch=pch,add=TRUE,col=color,cex=2)
}

vellipsemean <- function(Sigma,color,lty,lwd=1)
#draw an ellipse representing covariance matrix Sigma with specified color and linetype. Ellipse is drawn at Mahalanobis distance of 1.
{
    ellipses(acomp(c(1,1,1)),ilrvar2clr(Sigma),col=color,lty=lty,lwd=lwd)
}


simnoise <- function(model,df,reps,npoints,doplot="raw",s2,mu2,sigma2)
#simulate multinomial observation error, refit model, plot estimated parameters
#model is fitted model, df is real data frame, reps is number of replicate data frames to simulate, npoints is number of points in multinomial sampling. Argument doplot should be one of "raw" (plot parameters from each rep), "mean" (plot means), "none" (no plot). If it's "raw", then points will be added to a pre-existing plot
#returns list:
#trumu: true stationary mean
#trumusens: true sensitivity of stationary mean
#simumean: mean of stationary means from simulation
#simmusensmean: mean of sensitivities of stationary means from simulation
#truparms: true parameter values used for simulation
#simmeans: mean estimated parameter values from simulation))
{   
    B <- getB(model)
    Sigma <- estVar(model)
    initdata <- getinits(df)
    dcoef <- dim(coef(model))
    nparms <- dcoef[1]
    dcoef[1] <- dcoef[1]+2
    simcoef <- array(dim=c(dcoef,reps))
    simmu <- array(dim=c(2,reps))
    simmusens <- array(dim=c(2,reps))
#    if(!(doplot=="none")){
#        darkbgcol <- "black"
#        plotmodel(model,makelegend=TRUE,plotvariance=TRUE,bgcol=darkbgcol,lx=0.65,ly=1,lwd=1)
#    }
    colors <- c("grey","#56B4E9","#0072B2" , "#009E73", "#CC79A7","#F0E442","#E69F00","#D55E00")

    for(r in 1:reps){
        dfnoise <- simulateonemulti(nseries=initdata$nseries,slens=initdata$slens,inits=initdata$inits,npoints=npoints,model=model,B=B,Sigma=Sigma,df=df)
        mnoise <- fitbasemodel(dfnoise)
        simcoef[,,r] <- rbind(coef(mnoise),estVar(mnoise))
        simmu[,r] <- getmustar(model=mnoise,s2=s2,mu2=mu2,sigma2=sigma2,threat="Low")
        simmusens[,r] <- getmustarsens(model=mnoise,sigma2=sigma2)
        if(doplot=="raw"){
            for(i in 1:nparms){
                addpoint(i,mnoise,pch=16,color=colors[i])
            }
            varianceellipse(mnoise,color=adjustcolor("grey",0.3),lty="dashed",lwd=1)
        }
    }
    print("Coefficients from observed data:")
    truparms <- rbind(coef(model),estVar(model))
    print(truparms)

    simmeans <- apply(simcoef,c(1,2),mean)
    rownames(simmeans) <- c(rownames(coef(model)),"covz1","covz2")
    print("Mean coefficients from simulated data:")
    print(simmeans)
    print("Bias:")
    print(simmeans-truparms)

    trumu <- getmustar(model=model,s2=s2,mu2=mu2,sigma2=sigma2,threat="Low")
    trumusens <- getmustarsens(model=model,sigma2=sigma2)

    print("Stationary mean from observed data:")
    print(trumu)
    print("Mean stationary mean from simulated data:")
    simmumean <- apply(simmu,1,mean)
    print(simmumean)
    print("Sensitivity of stationary mean from observed data:")
    print(trumusens)
    print("Mean sensitivity of stationary mean from simulated data:")
    simmusensmean <- apply(simmusens,1,mean)
    print(simmusensmean)
    
#    if(doplot=="mean"){
#        for(i in 1:nparms){
#            addpointcoord(simmeans[i,],pch=3,color=colors[i])
#        }
#        vellipsemean(Sigma=matrix(simmeans[(nparms+1):(nparms+2),],nrow=2,ncol=2),color=adjustcolor("grey",0.7),lty="dotted",lwd=1)
#    }
    return(list(trumu=trumu,trumusens=trumusens,simmumean=simmumean,simmusensmean=simmusensmean,truparms=truparms,simmeans=simmeans))
}

arrowbias <- function(trumu,trumusens,simmumean,simmusensmean){
    nsteps <- 100
    mssm <- matrix(rep(trumusens,nsteps),nrow=nsteps,ncol=2,byrow=TRUE)
    mustarm <- matrix(rep(trumu,nsteps),nrow=nsteps,ncol=2,byrow=TRUE)
    sstseq <- seq(from=0,to=3.5,length.out=nsteps)
    mustarchange <- mustarm+sstseq*mssm

    smssm <- matrix(rep(simmusensmean,nsteps),nrow=nsteps,ncol=2,byrow=TRUE)
    smustarm <- matrix(rep(simmumean,nsteps),nrow=nsteps,ncol=2,byrow=TRUE)
    smustarchange <- smustarm+sstseq*smssm
 

    s60 <- sin(pi/3)
    c60 <- cos(pi/3)
 
    par(mar=c(0,2,0,2))
    par(xpd=TRUE)
    plot.acomp(c(1,1,1),type="n",labels=c("coral","algae","other")) #dummy plot to create graph
    mygrey <- colors()[320]
    isoProportionLines(at=0.5,col=mygrey)
    plot.acomp(ilrInv(rbind(trumu,mustarchange)),type="l",lwd=3,add=TRUE,lty="dashed")
    msc <- ilrInv(mustarchange)
    x <- msc[,2]+msc[,3]*c60
    y <- msc[,3]*s60
    nx <- length(x)
    arrows(x0=x[nx-1],y0=y[nx-1],x1=x[nx],y1=y[nx],lwd=3,length=0.1)
    plot.acomp(ilrInv(rbind(simmumean,smustarchange)),type="l",lwd=3,add=TRUE,lty="dotted",col="red")
    smsc <- ilrInv(smustarchange)
    sx <- smsc[,2]+smsc[,3]*c60
    sy <- smsc[,3]*s60
    snx <- length(sx)
    arrows(x0=sx[snx-1],y0=sy[snx-1],x1=sx[snx],y1=sy[snx],lwd=3,length=0.1,col="red")


    legend("topleft",bty="n",lwd=3,lty=c("dotted","dashed"),col=c("red","black"),legend=c("observation error","no observation error"),horiz=TRUE,seg.len=2.5)
}

#map of sites, color coded by threat level, outliers marked with x
#df is a data frame
#outliers is a logical vector, TRUE for outliers
makesitemap <- function(df,outliers){

    pts <- as.matrix(dfgbr[,c("long","lat")])
    mp <- map("worldHires",region="Australia", plot = FALSE)
    map(mp,xlim=c(140,155),ylim=c(-30,-5))
    #points(pts,col="blue",pch=16,cex=0.5)
    mycolors <- c("#56B4E9","#0072B2" , "#009E73", "#CC79A7","#F0E442","#E69F00","#D55E00")
    transcolors <- adjustcolor(mycolors,0.2)
    mypch <- rep(16,dim(df)[1])
    points(pts[dfgbr$threat=="Low",],pch=mypch[dfgbr$threat=="Low"],cex=0.7,col=adjustcolor("grey",0.1))
    points(pts[dfgbr$threat=="Medium",],pch=mypch[dfgbr$threat=="Medium"],cex=0.7,col=mycolors[5])
    points(pts[dfgbr$threat=="High",],pch=mypch[dfgbr$threat=="High"],cex=0.7,col=mycolors[6])
    points(pts[dfgbr$threat=="Very High",],pch=mypch[dfgbr$threat=="Very High"],cex=0.7,col=mycolors[7])
    points(pts[outliers==TRUE,],pch=4,cex=0.7,col=adjustcolor("black",0.2))
    map.scale(x=145, y=-10, ratio=FALSE, relwidth=0.2,cex=0.4)
    require(GISTools)
    north.arrow(xb=145, yb=-22, len=0.25, lab="N")
    detach("package:GISTools",unload=TRUE)
    legend("bottomleft",pch=c(16,16,16,16,4),col=c("grey",mycolors[5:7],"black"),legend=c("low threat","medium threat","high threat","very high threat","outlier"),bty="n")

}

#resids is an array of residuals
#outliers is a logical vector, TRUE for outliers
plotresidscatter <- function(resids,outliers){
#    plot(resids,xlab=expression(italic(p)[1]),ylab=expression(italic(p)[2]),asp=1,type="n")
    plot(resids,xlab="log(algae/coral)",ylab="log(other/geomean(algae,coral))",asp=1,type="n")
    points(resids[outliers==FALSE,],pch=16,col=adjustcolor("black",0.2))
    points(resids[outliers==TRUE,],pch=4,col=adjustcolor("red",0.5))
    legend("topright",pch=c(16,4),col=c("black","red"),legend=c("non-outlier","outlier"),bty="n")
}

#predicted vs observed with lowess line
#model: fitted model
#df: data frame
predvobs <- function(model,df){
    fits <- fitted.values(model)+cbind(df$o1,df$o2) #fitted increment, so add starting point
    backfit <- ilrInv(fits)
    backt1 <- ilrInv(cbind(df$c1,df$c2))
    backt <- ilrInv(cbind(df$o1,df$o2))

    par(mfrow=c(2,2))
    par(pty="s")
    plot(c(0,1),c(0,1),xlab="Observed coral",ylab="Predicted coral",xlim=c(0,1),ylim=c(0,1),asp=1,type="n",xaxs="i",yaxs="i")
    points(backt1[,1],backfit[,1],pch=16,col=adjustcolor("black",0.2))
    abline(a=0,b=1,col="black",lty="dashed")
    lines(lowess(backt1[,1],backfit[,1]),col="red")
    text(0.05,0.95,"A")

    plot(c(0,1),c(0,1),xlab="Observed algae",ylab="Predicted algae",xlim=c(0,1),ylim=c(0,1),asp=1,type="n",xaxs="i",yaxs="i")
    points(backt1[,2],backfit[,2],pch=16,col=adjustcolor("black",0.2))
    abline(a=0,b=1,col="black",lty="dashed")
    lines(lowess(backt1[,2],backfit[,2]),col="red")
    text(0.05,0.95,"B")

    plot(c(0,1),c(0,1),xlab="Observed other",ylab="Predicted other",xlim=c(0,1),ylim=c(0,1),asp=1,type="n",xaxs="i",yaxs="i")
    points(backt1[,3],backfit[,3],pch=16,col=adjustcolor("black",0.2))
    abline(a=0,b=1,col="black",lty="dashed")
    lines(lowess(backt1[,3],backfit[,3]),col="red")
    text(0.05,0.95,"C")
}

#label lab at top left of panel
textlabel <- function(lab){
    axl <- par("usr")
    text(axl[1]+0.05*(axl[2]-axl[1]),axl[4]-0.05*(axl[4]-axl[3]),lab)

}

residvexplan <- function(resids,fits,df)
#diagnostic plots of residuals
#resids and fits are residuals and fitted values respectively
#df is data frame containing explanatory variables
{
	par(mfrow=c(5,2))
        par(mar=c(4.5,4,1,1))

	plot(df$o1,resids[,1],xlab=expression(italic(x)[1](italic(t))),ylab="resid 1",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$o1,resids[,1]),col="red")
        textlabel("A")

	plot(df$o1,resids[,2],xlab=expression(italic(x)[1](italic(t))),ylab="resid 2",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$o1,resids[,2]),col="red")
        textlabel("B")
        
	plot(df$o2,resids[,1],xlab=expression(italic(x)[2](italic(t))),ylab="resid 1",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$o2,resids[,1]),col="red")
        textlabel("C")
        
	plot(df$o2,resids[,2],xlab=expression(italic(x)[2](italic(t))),ylab="resid 2",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$o2,resids[,2]),col="red")
        textlabel("D")
        
	plot(df$sstlag,resids[,1],xlab="lagged sst",ylab="resid 1",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$sstlag,resids[,1]),col="red")
        textlabel("E")
        
	plot(df$sstlag,resids[,2],xlab="lagged sst",ylab="resid 2",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$sstlag,resids[,2]),col="red")
        textlabel("F")
        
	plot(df$climlag,resids[,1],xlab="lagged climatology",ylab="resid 1",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$climlag,resids[,1]),col="red")
        textlabel("G")
        
	plot(df$climlag,resids[,2],xlab="lagged climatology",ylab="resid 2",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$climlag,resids[,2]),col="red")
        textlabel("H")
        
	plot(df$threat,resids[,1],xlab="threat",ylab="resid 1")
        textlabel("I")

	plot(df$threat,resids[,2],xlab="threat",ylab="resid 2")
        textlabel("J")

}

#residuals by year
residvyear <- function(resids,df){
	par(mfrow=c(2,1))
        par(mar=c(5,4,1,1))
	plot(df$year,resids[,1],xlab="year",ylab="resid 1",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$year,resids[,1]),col="red")
        textlabel("A")
        
	plot(df$year,resids[,2],xlab="year",ylab="resid 2",pch=16,col=adjustcolor("black",0.2))
	lines(lowess(df$year,resids[,2]),col="red")
        textlabel("B")
}
